/*
 *  TP-LINK TL-WR1041 v2 board support
 *
 *  Copyright (C) 2010-2012 Gabor Juhos <juhosg@openwrt.org>
 *  Copyright (C) 2011-2012 Anan Huang <axishero@foxmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License version 2 as published
 *  by the Free Software Foundation.
 */

#include <linux/pci.h>
#include <linux/phy.h>
#include <linux/platform_device.h>
#include <linux/ath9k_platform.h>
#include <linux/ar8216_platform.h>

#include <asm/mach-ath79/ar71xx_regs.h>

#include "common.h"
#include "dev-ap9x-pci.h"
#include "dev-eth.h"
#include "dev-gpio-buttons.h"
#include "dev-leds-gpio.h"
#include "dev-m25p80.h"
#include "dev-spi.h"
#include "dev-wmac.h"
#include "machtypes.h"

#define TL_WR1041NV2_GPIO_BTN_RESET	0
#define TL_WR1041NV2_GPIO_BTN_WPS		16

#define TL_WR1041NV2_GPIO_LED_SYSTEM	14
#define TL_WR1041NV2_GPIO_LED_LAN		11
#define TL_WR1041NV2_GPIO_LED_WLAN1	15
#define TL_WR1041NV2_GPIO_LED_WLAN2	13
#define TL_WR1041NV2_GPIO_LED_WLAN3	17

#define TL_WR1041NV2_KEYS_POLL_INTERVAL		20	/* msecs */
#define TL_WR1041NV2_KEYS_DEBOUNCE_INTERVAL	(3 * TL_WR1041NV2_KEYS_POLL_INTERVAL)

#define TL_WR1041NV2_PCIE_CALDATA_OFFSET	0x5000

static const char *tl_wr1041nv2_part_probes[] = {
	"tp-link",
	NULL,
};

static struct flash_platform_data tl_wr1041nv2_flash_data = {
	.part_probes	= tl_wr1041nv2_part_probes,
};

static struct gpio_led tl_wr1041nv2_leds_gpio[] __initdata = {
	{
		.name		= "huawei:blue:system",
		.gpio		= TL_WR1041NV2_GPIO_LED_SYSTEM,
		.active_low	= 1,
	}, {
		.name		= "huawei:green:lan",
		.gpio		= TL_WR1041NV2_GPIO_LED_LAN,
		.active_low	= 1,
	}, {
		.name		= "huawei:green:wlan1",
		.gpio		= TL_WR1041NV2_GPIO_LED_WLAN1,
		.active_low	= 1,
	}, {
		.name		= "huawei:green:wlan2",
		.gpio		= TL_WR1041NV2_GPIO_LED_WLAN2,
		.active_low	= 1,
	}, {
		.name		= "huawei:green:wlan3",
		.gpio		= TL_WR1041NV2_GPIO_LED_WLAN3,
		.active_low	= 1,
	}
};

static struct gpio_keys_button tl_wr1041nv2_gpio_keys[] __initdata = {
	{
		.desc		= "reset",
		.type		= EV_KEY,
		.code		= KEY_RESTART,
		.debounce_interval = TL_WR1041NV2_KEYS_DEBOUNCE_INTERVAL,
		.gpio		= TL_WR1041NV2_GPIO_BTN_RESET,
		.active_low	= 1,
	}, {
		.desc		= "wps",
		.type		= EV_KEY,
		.code		= KEY_RESTART,
		.debounce_interval = TL_WR1041NV2_KEYS_DEBOUNCE_INTERVAL,
		.gpio		= TL_WR1041NV2_GPIO_BTN_WPS,
		.active_low	= 1,
	}
};

static void __init tl_wr1041nv2_setup(void)
{
	u8 *mac = (u8 *) KSEG1ADDR(0x1f01fc00);
	u8 *ee = (u8 *) KSEG1ADDR(0x1fff1000);

	ath79_register_m25p80(&tl_wr1041nv2_flash_data);

	ath79_register_leds_gpio(-1, ARRAY_SIZE(tl_wr1041nv2_leds_gpio),
				 tl_wr1041nv2_leds_gpio);
	ath79_register_gpio_keys_polled(-1, TL_WR1041NV2_KEYS_POLL_INTERVAL,
					 ARRAY_SIZE(tl_wr1041nv2_gpio_keys),
					 tl_wr1041nv2_gpio_keys);
	ath79_register_wmac(ee, mac);

	ath79_setup_ar934x_eth_cfg(AR934X_ETH_CFG_MII_GMAC0 |
				   AR934X_ETH_CFG_SW_ONLY_MODE);

	ath79_register_mdio(0, 0x0);
	ath79_eth0_data.reset_bit = AR71XX_RESET_GE0_MAC |
				    AR71XX_RESET_GE0_PHY;

	ath79_init_mac(ath79_eth0_data.mac_addr, mac, 1);


	/* GMAC0 is connected to an AR8327 switch */
	ath79_eth0_data.phy_if_mode = PHY_INTERFACE_MODE_MII;
	ath79_eth0_data.phy_mask = BIT(0);
	ath79_eth0_data.mii_bus_dev = &ath79_mdio0_device.dev;
	ath79_eth0_pll_data.pll_1000 = 0x00001000;
	ath79_register_eth(0);
}

MIPS_MACHINE(ATH79_MACH_TL_WR1041N_V2, "TL-WR1041N-v2",
	     "HUAWEI WS322", tl_wr1041nv2_setup);
